#include <iostream>
#include <cstdio>
#include <cstring>
#include <SDL/SDL.h>

#include "sprite_handler.h"

enum DIRECTION {LEFT, RIGHT, UP, DOWN};

void move_badguy(Sprite_Handler& handler);

int main(int argc, char **argv)
{

  SDL_Event event;

  bool exit = false;
  bool active = true;
  Uint8 *key_state;
  DIRECTION p_dir;
  
 // short x = 15;
 // short y = 15;

  Sprite_Handler sprite_handler; //sprite handler that takes care of processing all sprites in
                         //the game.

  if (!sprite_handler.load_bitmaps())
  {
    std::cout << "\nERROR: Program could not load necessary image files!\n";
    return -1;
  }
  
  //the main loop
  while (!exit)
  {
    move_badguy(sprite_handler);
  
    if (active) //if it's not minimized...
    {

    
      if (!sprite_handler.draw_scene()) //there's a collision
      {
        switch(p_dir)
	{
	  case LEFT:
	    sprite_handler.player_obj.set_x(25);
	    break;
	    
	  case RIGHT:
            sprite_handler.player_obj.set_x(-25);
	    break;
	    
	  case UP:
	    sprite_handler.player_obj.set_y(25);
	    break;
	    
	  case DOWN:
	    sprite_handler.player_obj.set_y(-25);
	    break;
	}
      }
    }
  
    while (SDL_PollEvent(&event))
    {
      switch(event.type)
      {
        case SDL_ACTIVEEVENT: //if something happens like minimizing or maximixing.
	  if (event.active.state & SDL_APPACTIVE)
	  {
	    if (event.active.gain) //if surface gains activity...
	      active = true;
	    else
	      active = false;     //if surface loses activity....
	  }
	  break;
      
        case SDL_QUIT:
	  exit = true;
	  std::cout << "\nProgram recieved quit request from event handler.\n";
	  break;
	  
	case SDL_KEYDOWN:
	  
	  if (event.key.keysym.sym == SDLK_ESCAPE)
	    exit = true;
	    
	  if (event.key.keysym.sym == SDLK_q)
	    exit = true;
	  	  
	  break;
	 
      }
    }
    
    //keystates are done outside of poll events so we can work as the key is being
    //held down. (aka they don't have to keep tapping it)

    key_state = SDL_GetKeyState(NULL);

    if (key_state[SDLK_LEFT])
    {
      p_dir = LEFT;
      sprite_handler.player_obj.set_x(-5);
    }
      
    if (key_state[SDLK_RIGHT])
    {
      p_dir = RIGHT;
      sprite_handler.player_obj.set_x(5);
    }
      
    if (key_state[SDLK_UP])
    {
      p_dir = UP;
      sprite_handler.player_obj.set_y(-5);
    }
      
    if (key_state[SDLK_DOWN])
    {
      p_dir = DOWN;
      sprite_handler.player_obj.set_y(5);
    }
	    

  }
  
  SDL_Quit();
  std::cout << "Terminating program...\n";
  return 0;
}

void move_badguy(Sprite_Handler& handler)
{

  //this is basically a debug function to make the bad guy chase the player.

  if (handler.enemy_obj.get_x() < handler.player_obj.get_x())  //if players to the right.
    handler.enemy_obj.set_x(1);                               //move right.
    
  if (handler.enemy_obj.get_x() > handler.player_obj.get_x())  //if players to the left
    handler.enemy_obj.set_x(-1);                               //move left
  
  if (handler.enemy_obj.get_y() < handler.player_obj.get_y())  //if players above.
    handler.enemy_obj.set_y(1);                               //move up.

  if (handler.enemy_obj.get_y() > handler.player_obj.get_y())  //if players below.
    handler.enemy_obj.set_y(-1);                               //move down.
}
