
// sprite class implementation file.

#include "sprite_handler.h"

Sprite_Handler::Sprite_Handler() : player_obj("./images/p_small.bmp", 250, 150),
bg_obj("./images/background_test_small.bmp", 0, 0),
enemy_obj("./images/pepper_enemy.bmp", 290, 200)
{

  //takes in command line to set up the screen or defaults if there is none.
  //returns a false on error and true on success
  
  char *title_string = "RPG GFX Alpha";
  char *icon_string = "RPG GFX Alpha - Minimized";
  char try_again;
  
  //clear console for debugging output
  system("reset");
  
  
  //initializing SDL for graphics abilities...
  std::cout << "Initializing SDL...";
  if (SDL_Init(SDL_INIT_VIDEO) < 0)
  {
    fprintf(stderr, "\nERROR: Could not initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }
  
  else
  {
    std::cout << "Done!\n";
    atexit(SDL_Quit);
  }
       
  std::cout << "Setting up window for application on hardware surface...";
  
  
  //set up a simple screen that supports any bitdepth on the hardaware surface.
  //if that fails, it will prompt to try the software surface. If this fails the program
  //bails out.
  
  screen = SDL_SetVideoMode(640, 480, 16, SDL_HWSURFACE | SDL_ANYFORMAT | SDL_DOUBLEBUF);// | SDL_FULLSCREEN);
  if (!screen)
  {
    fprintf(stderr, "\nERROR: Could not set up screen on hardware surface: %s\n", SDL_GetError());
    std::cout << "\nTry again using a software surface? (Warning: Program may run slower!)[y/n]: ";
    std::cin >> try_again;
    
    if ((try_again == 'y') || (try_again == 'Y'))
    {
      screen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE | SDL_ANYFORMAT);
      if (!screen)
      {
        fprintf(stderr, "\nERROR: Could not set up screen on software surface: %s\n", SDL_GetError());
	std::cout << "Out of options, program must terminate. Please report this error to website!\n";
        exit(1);
      }
    }
    else
    {
      std::cout << "Not trying software surface, program terminating...\n";
      exit(1);
    }  
  }
  
  else
    std::cout << "Done!\n";
    
  //setting window's caption:
  std::cout << "Setting window caption and minimized caption...";
  SDL_WM_SetCaption(title_string, icon_string);
  std::cout << "Done!\n";
  std::cout << "Testing captions...\n";
  SDL_WM_GetCaption(&title_string, &icon_string);
  if (title_string)
    std::cout << "Got title_string: " << title_string << std::endl;
  if (icon_string)
    std::cout << "Got icon_string: " << icon_string << std::endl;
  
  //clearing the screen
  std::cout << "Clearing the screen so it's fresh...";
  SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format,0,0,0));
  std::cout << "Done!\n";
  
  //setting transparency color key
  std::cout << "Setting transparence color to FF00FF...";
  colorKey = SDL_MapRGB(screen->format, 255, 0, 255);
  
  if (colorKey > 0)
    std::cout << "Done!\n";
  else
  {
    std::cout << "\n\nWARNING: SDL_MapRGB(SDL_PixelFormat*, Uint8, Uint8, Uint8) returned a zero "
                 "or negative value. Continuing program, but it may not run correctly...\n";
  }

  std::cout << "Setting transparency color key to sprites...";
  player_obj.set_colorKey(colorKey);
  enemy_obj.set_colorKey(colorKey);
  std::cout << "Done!\n";
  
  //removing the cursor...
  std::cout << "Removing the cursor...";
  SDL_ShowCursor(false);
  std::cout << "Done!\n";
  
  std::cout << "Initialization complete!\nStarting program...\n";

    


}

Sprite_Handler::~Sprite_Handler()
{
  //for some reason freeing the screen surface is causing the game not to terminate...

 // SDL_FreeSurface(screen);
  
  SDL_Quit();
}

bool Sprite_Handler::load_bitmaps(void)
{
  //loads bitmaps used in the game into memory. 
  //possibly in the future this will be re-worked so that it loads bitmaps specific
  //to the area that the player is in.
  
  //player image is automatically loaded in the constructor of the class.
  //all other images "loaded on the fly" are done here.
  
  
  /*test to see if it loaded correctly (model for future, working one in player_sprite class)
  
  if (!player_info.player_surface)
    return false;
    
  else  //if successful set the transparency color and continue.
  { 
    SDL_SetColorKey(player_obj.get_image(0), SDL_SRCCOLORKEY | SDL_RLEACCEL, colorKey);
    return true;
  }
  */
  
  return true; //just to keep the compiler from complaining.
}

bool Sprite_Handler::draw_scene(void)
{
  //draws the images to the screen and locks it if necessary before doing so.
  
  //locking if necessary...
  if (SDL_MUSTLOCK(screen))
  {
    if (SDL_LockSurface(screen) < 0)
      return false;
  }
  
  //clearing the screen
 // SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format,0,0,0)); //draws blank screen.
  draw_sprite(bg_obj.get_image(0), bg_obj.get_x(), bg_obj.get_y());
  draw_sprite(enemy_obj.get_image(0), enemy_obj.get_x(), enemy_obj.get_y());
  draw_sprite(player_obj.get_image(0), player_obj.get_x(), player_obj.get_y());

    //needs more testing
  //draw_part_sprite(player_obj.get_image(0), player_obj.get_x(), player_obj.get_y(), player_obj.get_w(), player_obj.get_h()); //draws bitmap test sprite
 
  
  //unlock the screen in need be:
  if (SDL_MUSTLOCK(screen))
    SDL_UnlockSurface(screen);
    
  //flip the buffers so we see the image.
  SDL_Flip(screen);
  
  if (collision_detection(enemy_obj, player_obj))
    return false;
    
  return true;
}

bool Sprite_Handler::collision_detection(Sprite &in1, Sprite &in2)
{
  //sprites are passed in via reference so we know who we're talking about.

  SDL_Rect rect1;
  SDL_Rect rect2;
  
  //setting the rect information based on the surfaces sent to be checked.
  rect1.x = in1.get_x();
  rect1.y = in1.get_y();
  rect1.h = in1.get_h();
  rect1.w = in1.get_w();
  
  rect2.x = in2.get_x();
  rect2.y = in2.get_y();
  rect2.h = in2.get_h();
  rect2.w = in2.get_w();


  //check to see if boxes are touching on any side...
  //these statements would be false if they are.
  
  if(rect2.x + rect2.w < rect1.x)	 //rect2's right side not touching rect1's...
    return false;	

  if(rect2.x > rect1.x + rect1.w)	 //rect1's right side not touching rect2's...
    return false;

  if(rect2.y + rect2.h < rect1.y)        //rect 2's top side not touching rect1's...
    return false;
	
  if(rect2.y > rect1.y + rect1.h)        //rect1's top side not touching rect2's...
    return false;


  //if they haven't been thrown back yet, they must be touching.
  return true;
}

void Sprite_Handler::draw_sprite(SDL_Surface *surface, short x, short y)
{
  SDL_Rect destination;
  
  //set draw locations...
  destination.x = x;
  destination.y = y;
  
  //draw given surface, in entirety, on the main screen, at the destination given.
  SDL_BlitSurface(surface, NULL, screen, &destination);
}

void Sprite_Handler::draw_part_sprite(SDL_Surface *surface, short x, short y, short width, short height)
{
  //still in testing. Needs to mask old!

  SDL_Rect destination_img;
  SDL_Rect destination_surface;
  
  destination_img.x = 0; //x;
  destination_img.y = 0; //y;
  destination_img.w = width; //width;
  destination_img.h = height; //height;
  
  destination_surface.x = x; //x;
  destination_surface.y = y; //y;
  
  SDL_BlitSurface(surface, &destination_img, bg_obj.get_image(0), &destination_surface);
}


